'use strict';

var express = require('express');
var HTTPStatus = require('http-status');
var errors = require('./errors');

var _require = require('./utils/helpers'),
    formatResponse = _require.formatResponse,
    expect = _require.expect,
    depreciated = _require.depreciated;

var _require2 = require('./utils/auth'),
    authPopulate = _require2.authPopulate,
    tokenPopulate = _require2.tokenPopulate;

var UserResource = require('./user/resource');

/**
 * Parse the body of the requests.
 */
function parseRequestBody(app) {
  app.use(express.json());
  app.use(express.urlencoded({ extended: true }));
}

/**
 * Catch any requests or errors which aren't in the api.
 */
function handleErrors(app, debug) {
  return app.use(function (req, res, next) {
    var error = new errors.NotFoundResponse({ message: 'Request address does not exist on the api.' });
    next(error);
  }).use(function (err, req, res, next) {
    res.status(err.status || HTTPStatus.INTERNAL_SERVER_ERROR).json(formatResponse(err, debug));
    next();
  });
}

/**
 * Add a health check route.
 */
function environmentCheck(app) {
  var response = formatResponse({
    data: {
      environment: process.env.NODE_ENV
    }
  });
  app.get('/', function (req, res) {
    res.status(response.status || HTTPStatus.INTERNAL_SERVER_ERROR).json(response);
  });
}

/**
 * Attach multiple resources to a single app instance and provide authentication.
 *
 * @param {function} app the express app instance.
 * @param {array} resources an array of the hobson resources to attach to the app.
 * @param {string} secret a unique, random secret string used in authenticating users and tokens.
 * @param {boolean} status this will add a index route which returns the app environment.
 * @param {boolean} debug determines if the app should return stack traces in errors.
 * @param {function} before a function used to configure the app before attaching resources (not recommended to change).
 * @param {function} after a function used to configure the app after attaching resources.
 */
module.exports = function attach(_ref) {
  var app = _ref.app,
      resources = _ref.resources,
      secret = _ref.secret,
      _ref$status = _ref.status,
      status = _ref$status === undefined ? true : _ref$status,
      _ref$debug = _ref.debug,
      debug = _ref$debug === undefined ? false : _ref$debug,
      before = _ref.before,
      after = _ref.after,
      token = _ref.token;

  if (typeof app !== 'function' || typeof app.use !== 'function') {
    throw new errors.BreakingResponse({ message: 'Parameter "app" must be an express app instance.' });
  }
  if (!resources || !Array.isArray(resources)) {
    throw new errors.BreakingResponse({ message: 'Parameter "resources" must be an array of resources.' });
  }
  if (typeof secret !== 'string') {
    throw new errors.BreakingResponse({ message: 'Parameter "secret" must be a random string used to authenticate requests.' });
  }
  depreciated({ name: 'token', value: token });
  expect({ name: 'status', value: status, type: 'boolean' });
  expect({ name: 'debug', value: debug, type: 'boolean' });
  expect({ name: 'before', value: before, type: 'function', optional: true });
  expect({ name: 'after', value: after, type: 'function', optional: true });
  if (before) {
    before(app);
  } else {
    parseRequestBody(app);
  }
  app.use(tokenPopulate({ secret: secret }));
  resources.filter(function (source) {
    return source instanceof UserResource;
  }).forEach(function (source) {
    app.use(authPopulate({ User: source.model, secret: secret }));
    source.option({ secret: secret });
  });
  resources.forEach(function (resource) {
    return resource.attach(app);
  });
  if (status) {
    environmentCheck(app);
  }
  if (after) {
    after(app);
  }
  handleErrors(app, debug);
};